@@ -182,6 +182,7 @@ e_book_backend_scalix_load_source (EBookBackend * backend,
     EBookBackendScalix *bs;
     EBookBackendScalixPrivate *priv;
     ScalixContainer *container;
+    const char *prop_ssl;
     gchar *uri;
 
     bs = E_BOOK_BACKEND_SCALIX (backend);
@@ -193,6 +194,16 @@ e_book_backend_scalix_load_source (EBookBackend * backend,
     }
 
     uri = e_source_get_uri (source);
+
+    if ((prop_ssl = e_source_get_property (source, "use_ssl"))) {
+		CamelURL *url;
+
+		url = camel_url_new (uri, NULL);
+		camel_url_set_param (url, "use_ssl", prop_ssl);
+		g_free (uri);
+		uri = camel_url_to_string (url, 0);
+    }
+
     container = scalix_container_open (uri);
     g_free (uri);
 
