@@ -339,78 +339,6 @@ scalix_get_name (CamelService *service, gboolean brief)
 }
 
 
-static gboolean
-connect_to_server_process (CamelSCALIXEngine *engine, const char *format, CamelException *ex)
-{
-	CamelService *service = engine->service;
-	CamelStream *stream;
-	const char *start;
-	GString *exec;
-	
-	exec = g_string_new ("");
-	start = format;
-	
-	while (*format) {
-		register char ch = *format++;
-		
-		if (ch == '%') {
-			g_string_append_len (exec, start, format - start - 1);
-			
-			switch (*format) {
-			case '%':
-				/* literal % */
-				g_string_append_c (exec, '%');
-				break;
-			case 'h':
-				g_string_append (exec, service->url->host);
-				break;
-			case 'u':
-				g_string_append (exec, service->url->user);
-				break;
-			default:
-				g_warning ("unknown formatter '%%%c'", *format);
-				g_string_append_c (exec, '%');
-				g_string_append_c (exec, *format);
-				break;
-			}
-			
-			format++;
-			
-			start = format;
-		}
-	}
-	
-	g_string_append (exec, start);
-	
-	stream = camel_stream_process_new ();
-	
-	if (camel_stream_process_connect ((CamelStreamProcess *) stream, exec->str, NULL) == -1) {
-		if (errno == EINTR)
-			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
-					     _("Connection cancelled"));
-		else
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-					      _("Could not connect with command \"%s\": %s"),
-					      exec->str, g_strerror (errno));
-		
-		camel_object_unref (stream);
-		g_string_free (exec, TRUE);
-		return FALSE;
-	}
-	
-	g_string_free (exec, TRUE);
-	
-	if (camel_scalix_engine_take_stream (engine, stream, ex) == -1)
-		return FALSE;
-	
-	if (camel_scalix_engine_capability (engine, ex) == -1)
-		return FALSE;
-	
-	camel_scalix_store_summary_set_capabilities (((CamelSCALIXStore *) service)->summary, engine->capa);
-	
-	return TRUE;
-}
-
 enum {
 	MODE_CLEAR,
 	MODE_SSL,
@@ -426,14 +354,15 @@ static gboolean
 connect_to_server (CamelSCALIXEngine *engine, struct addrinfo *ai, int ssl_mode, CamelException *ex)
 {
 	CamelService *service = engine->service;
+	CamelSockOptData sockopt;
 	CamelStream *tcp_stream;
 	int ret;
 #ifdef HAVE_SSL
 	CamelSCALIXCommand *ic;
 	int id;
 #endif
-	
-	
+
+
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
 		if (ssl_mode == MODE_TLS) {
@@ -445,13 +374,13 @@ connect_to_server (CamelSCALIXEngine *engine, struct addrinfo *ai, int ssl_mode,
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				      _("Could not connect to %s: %s"),
 				      service->url->host, _("SSL unavailable"));
-		
+
 		return FALSE;
 #endif /* HAVE_SSL */
 	} else {
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
-	
+
 	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
@@ -461,38 +390,47 @@ connect_to_server (CamelSCALIXEngine *engine, struct addrinfo *ai, int ssl_mode,
 					      _("Could not connect to %s: %s"),
 					      service->url->host,
 					      g_strerror (errno));
-		
+
 		camel_object_unref (tcp_stream);
-		
+
 		return FALSE;
 	}
-	
+
+	/* set some socket options to better tailor the connection to our needs */
+	sockopt.option = CAMEL_SOCKOPT_NODELAY;
+	sockopt.value.no_delay = TRUE;
+	camel_tcp_stream_setsockopt ((CamelTcpStream *) tcp_stream, &sockopt);
+
+	sockopt.option = CAMEL_SOCKOPT_KEEPALIVE;
+	sockopt.value.keep_alive = TRUE;
+	camel_tcp_stream_setsockopt ((CamelTcpStream *) tcp_stream, &sockopt);
+
 	if (camel_scalix_engine_take_stream (engine, tcp_stream, ex) == -1)
 		return FALSE;
-	
+
 	if (camel_scalix_engine_capability (engine, ex) == -1)
 		return FALSE;
-	
+
 	camel_scalix_store_summary_set_capabilities (((CamelSCALIXStore *) service)->summary, engine->capa);
-	
+
 	if (ssl_mode != MODE_TLS) {
 		/* we're done */
 		return TRUE;
 	}
-	
+
 #ifdef HAVE_SSL
 	if (!(engine->capa & CAMEL_SCALIX_CAPABILITY_STARTTLS)) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Failed to connect to IMAP server %s in secure mode: %s"),
 				      service->url->host, _("SSL negotiations failed"));
-		
+
 		return FALSE;
 	}
-	
+
 	ic = camel_scalix_engine_prequeue (engine, NULL, "STARTTLS\r\n");
 	while ((id = camel_scalix_engine_iterate (engine)) < ic->id && id != -1)
 		;
-	
+
 	if (id == -1 || ic->result != CAMEL_SCALIX_RESULT_OK) {
 		if (ic->result != CAMEL_SCALIX_RESULT_OK) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -501,20 +439,20 @@ connect_to_server (CamelSCALIXEngine *engine, struct addrinfo *ai, int ssl_mode,
 		} else {
 			camel_exception_xfer (ex, &ic->ex);
 		}
-		
+
 		camel_scalix_command_unref (ic);
-		
+
 		return FALSE;
 	}
-	
+
 	camel_scalix_command_unref (ic);
-	
+
 	return TRUE;
 #else
 	camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 			      _("Failed to connect to IMAP server %s in secure mode: %s"),
 			      service->url->host, _("SSL is not available in this build"));
-	
+
 	return FALSE;
 #endif /* HAVE_SSL */
 }
@@ -538,14 +476,10 @@ connect_to_server_wrapper (CamelSCALIXEngine *engine, CamelException *ex)
 	CamelService *service = engine->service;
 	struct addrinfo *ai, hints;
 	const char *ssl_mode;
-	const char *command;
 	int mode, ret, i;
 	const char *port;
 	char *serv;
-	
-	if ((command = camel_url_get_param (service->url, "command")))
-		return connect_to_server_process (engine, command, ex);
-	
+
 	if ((ssl_mode = camel_url_get_param (service->url, "use_ssl"))) {
 		for (i = 0; ssl_options[i].value; i++)
 			if (!strcmp (ssl_options[i].value, ssl_mode))
@@ -558,13 +492,13 @@ connect_to_server_wrapper (CamelSCALIXEngine *engine, CamelException *ex)
 		serv = "imap";
 		port = "143";
 	}
-	
+
 	if (service->url->port) {
 		serv = g_alloca (16);
 		sprintf (serv, "%d", service->url->port);
 		port = NULL;
 	}
-	
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
@@ -575,7 +509,7 @@ connect_to_server_wrapper (CamelSCALIXEngine *engine, CamelException *ex)
 	}
 	if (ai == NULL)
 		return FALSE;
-	
+
 	if (!(ret = connect_to_server (engine, ai, mode, ex)) && mode == MODE_SSL) {
 		camel_exception_clear (ex);
 		ret = connect_to_server (engine, ai, MODE_TLS, ex);
@@ -583,9 +517,9 @@ connect_to_server_wrapper (CamelSCALIXEngine *engine, CamelException *ex)
 		camel_exception_clear (ex);
 		ret = connect_to_server (engine, ai, MODE_CLEAR, ex);
 	}
-	
+
 	camel_freeaddrinfo (ai);
-	
+
 	return ret;
 }
 
@@ -632,36 +566,40 @@ scalix_try_authenticate (CamelSCALIXEngine *engine, gboolean reprompt, const cha
 {
 	CamelService *service = engine->service;
 	CamelSession *session = service->session;
+	CamelServiceAuthType *mech = NULL;
 	CamelSasl *sasl = NULL;
 	CamelSCALIXCommand *ic;
 	int id;
-	
-	if (!service->url->passwd) {
+
+	if (service->url->authmech) {
+		mech = g_hash_table_lookup (engine->authtypes,
+					    service->url->authmech);
+	}
+
+	if ((!mech || (mech && mech->need_password)) && !service->url->passwd) {
 		guint32 flags = CAMEL_SESSION_PASSWORD_SECRET;
 		char *prompt;
-		
+
 		if (reprompt)
 			flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-		
+
 		prompt = g_strdup_printf (_("%sPlease enter the IMAP password for %s on host %s"),
 					  errmsg ? errmsg : "",
 					  service->url->user,
 					  service->url->host);
-		
+
 		service->url->passwd = camel_session_get_password (session, service, NULL, prompt, "password", flags, ex);
-		
+
 		g_free (prompt);
-		
+
 		if (!service->url->passwd)
 			return FALSE;
 	}
-	
+
 	if (service->url->authmech) {
-		CamelServiceAuthType *mech;
-		
-		mech = g_hash_table_lookup (engine->authtypes, service->url->authmech);
+
 		sasl = camel_sasl_new ("imap", mech->authproto, service);
-		
+
 		ic = camel_scalix_engine_prequeue (engine, NULL, "AUTHENTICATE %s\r\n", service->url->authmech);
 		ic->plus = sasl_auth;
 		ic->user_data = sasl;
@@ -669,31 +607,31 @@ scalix_try_authenticate (CamelSCALIXEngine *engine, gboolean reprompt, const cha
 		ic = camel_scalix_engine_prequeue (engine, NULL, "LOGIN %S %S\r\n",
 						  service->url->user, service->url->passwd);
 	}
-	
+
 	while ((id = camel_scalix_engine_iterate (engine)) < ic->id && id != -1)
 		;
-	
+
 	if (sasl != NULL)
 		camel_object_unref (sasl);
-	
+
 	if (id == -1 || ic->status == CAMEL_SCALIX_COMMAND_ERROR) {
 		/* unrecoverable error */
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
-		
+
 		return FALSE;
 	}
-	
+
 	if (ic->result != CAMEL_SCALIX_RESULT_OK) {
 		camel_scalix_command_unref (ic);
-		
+
 		/* try again */
-		
+
 		return TRUE;
 	}
-	
+
 	camel_scalix_command_unref (ic);
-	
+
 	return FALSE;
 }
 
@@ -705,46 +643,49 @@ scalix_reconnect (CamelSCALIXEngine *engine, CamelException *ex)
 	gboolean reprompt = FALSE;
 	char *errmsg = NULL;
 	CamelException lex;
-	
+
 	if (!connect_to_server_wrapper (engine, ex))
 		return FALSE;
-	
+
+	if (engine->state != CAMEL_SCALIX_ENGINE_AUTHENTICATED) {
 #define CANT_USE_AUTHMECH (!(mech = g_hash_table_lookup (engine->authtypes, service->url->authmech)))
-	if (service->url->authmech && CANT_USE_AUTHMECH) {
-		/* Oops. We can't AUTH using the requested mechanism */
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
-				      _("Cannot authenticate to IMAP server %s using %s"),
-				      service->url->host, service->url->authmech);
-		
-		return FALSE;
-	}
-	
-	camel_exception_init (&lex);
+		if (service->url->authmech && CANT_USE_AUTHMECH) {
+			/* Oops. We can't AUTH using the requested mechanism */
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+					      _("Cannot authenticate to IMAP server %s using %s"),
+					      service->url->host, service->url->authmech);
 
-	if (camel_scalix_id (engine, ex) == -1) 
-		return FALSE;
+			return FALSE;
+		}
 
-	while (scalix_try_authenticate (engine, reprompt, errmsg, &lex)) {
+		camel_exception_init (&lex);
+
+		if (camel_scalix_id (engine, ex) == -1)
+			return FALSE;
+
+		while (scalix_try_authenticate (engine, reprompt, errmsg, &lex)) {
+			g_free (errmsg);
+			errmsg = g_strdup (lex.desc);
+			camel_exception_clear (&lex);
+			g_free (service->url->passwd);
+			service->url->passwd = NULL;
+			reprompt = TRUE;
+		}
 		g_free (errmsg);
-		errmsg = g_strdup (lex.desc);
-		camel_exception_clear (&lex);
-		g_free (service->url->passwd);
-		service->url->passwd = NULL;
-		reprompt = TRUE;
-	}
-	g_free (errmsg);
-	
-	if (camel_exception_is_set (&lex)) {
-		camel_exception_xfer (ex, &lex);
-		return FALSE;
+
+		if (camel_exception_is_set (&lex)) {
+			camel_exception_xfer (ex, &lex);
+			return FALSE;
+		}
+
 	}
 
-	if (camel_scalix_id (engine, ex) == -1) 
+	if (camel_scalix_id (engine, ex) == -1)
 		return FALSE;
 
 	if (camel_scalix_engine_namespace (engine, ex) == -1)
 		return FALSE;
-	
+
 	camel_scalix_store_summary_set_namespaces (((CamelSCALIXStore *) service)->summary, &engine->namespaces);
 
 	return TRUE;
@@ -755,17 +696,27 @@ scalix_connect (CamelService *service, CamelException *ex)
 {
 	CamelSCALIXStore *store = (CamelSCALIXStore *) service;
 	gboolean retval;
-	
+
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
 		return TRUE;
-	
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (service, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (service, connect_lock);
+#endif
+	
 	if (store->engine->state == CAMEL_SCALIX_ENGINE_DISCONNECTED)
 		retval = scalix_reconnect (store->engine, ex);
 	else
 		retval = TRUE;
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (service, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (service, connect_lock);
-	
+#endif
+
 	return retval;
 }
 
@@ -779,7 +730,11 @@ scalix_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
 		return TRUE;
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
+#endif
 	if (clean && store->engine->state != CAMEL_SCALIX_ENGINE_DISCONNECTED) {
 		ic = camel_scalix_engine_queue (store->engine, NULL, "LOGOUT\r\n");
 		while ((id = camel_scalix_engine_iterate (store->engine)) < ic->id && id != -1)
@@ -787,7 +742,12 @@ scalix_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 		
 		camel_scalix_command_unref (ic);
 	}
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 	
 	return 0;
 }
@@ -804,10 +764,20 @@ scalix_query_auth_types (CamelService *service, CamelException *ex)
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
 		return NULL;
-	
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
+#endif
+
 	connected = connect_to_server_wrapper (store->engine, ex);
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 	if (!connected)
 		return NULL;
 	
@@ -881,81 +851,91 @@ scalix_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Ca
 	int create;
 	int id, i;
 	guint32 folder_flags;
-	
-	CAMEL_SERVICE_LOCK (store, connect_lock);
 
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_LOCK (store, connect_lock);
+#endif
 	/* Scalix Control Folder */
 	if (g_str_equal (folder_name, "#ScalixControl")) {
 		folder = camel_scalix_cfolder_new (store);
 
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 		return folder;
 	}
-	
+
 	folder_flags = 0;
 	summary = CAMEL_STORE_SUMMARY (CAMEL_SCALIX_STORE (store)->summary);
-	
+
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		if ((flags & CAMEL_STORE_FOLDER_CREATE) != 0) {
-			camel_exception_set (ex, 
-					     CAMEL_EXCEPTION_SYSTEM, 
+			camel_exception_set (ex,
+					     CAMEL_EXCEPTION_SYSTEM,
 					     _("Cannot create IMAP folders in offline mode."));
 		} else {
 			CamelStoreInfo *si;
-			/* TODO: test me */	
+			/* TODO: test me */
 			/* we need to retrive the folder flags from the
-			 * summary */	
+			 * summary */
 			si = camel_store_summary_path (summary,
 						       folder_name);
-			
+
 			if (si != NULL) {
 				folder_flags = si->flags;
 				camel_store_summary_info_free (summary, si);
 			}
-			
+
 			folder = camel_scalix_folder_new (store,
 							  folder_name,
 							  folder_flags,
 							  ex);
-			
+
 		}
-		
+
+#if EAPI_CHECK_VERSION (2, 10)
+		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 		CAMEL_SERVICE_UNLOCK (store, connect_lock);
-		
+#endif
 		return folder;
 	}
-	
+
 	/* make sure the folder exists - try LISTing it? */
 	utf7_name = scalix_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_scalix_engine_queue (engine, NULL, "LIST \"\" %S\r\n", utf7_name);
 	camel_scalix_command_register_untagged (ic, "LIST", camel_scalix_untagged_list);
 	ic->user_data = array = g_ptr_array_new ();
 	g_free (utf7_name);
-	
+
 	while ((id = camel_scalix_engine_iterate (engine)) < ic->id && id != -1)
 		;
-	
+
 	if (id == -1 || ic->status != CAMEL_SCALIX_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
 		g_ptr_array_free (array, TRUE);
 		goto done;
 	}
-	
-	create = array->len == 0;	
+
+	create = array->len == 0;
 	for (i = 0; i < array->len; i++) {
 		list = array->pdata[i];
-		
+
 		if (g_str_equal (list->name, folder_name)) {
 			folder_flags = list->flags;
 		}
-		
+
 		g_free (list->name);
 		g_free (list);
 	}
-	
+
 	g_ptr_array_free (array, TRUE);
-	
+
 	if (ic->result != CAMEL_SCALIX_RESULT_OK) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot get folder `%s' on IMAP server %s: Unknown"),
@@ -1006,8 +986,12 @@ scalix_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Ca
 	folder = camel_scalix_folder_new (store, folder_name, folder_flags, ex);
 
  done:
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
-	
+#endif	
 	return folder;
 }
 
@@ -1017,14 +1001,14 @@ scalix_get_trash (CamelStore *camel_store, CamelException *ex)
 {
 	CamelSCALIXStore *store = (CamelSCALIXStore *) camel_store;
 	CamelFolder *folder;
-	
+
 	g_assert (store->trash_full_name);
-		
+
 	folder = camel_store_get_folder (camel_store,
-					 store->trash_full_name, 
-					 0, 
+					 store->trash_full_name,
+					 0,
 					 ex);
-	
+
 	return folder;
 }
 
@@ -1032,24 +1016,24 @@ static gboolean
 scalix_check_foldername (const char *folder_name, char sep)
 {
     const char *c;
-   
+
     if (!folder_name)
         return FALSE;
-   
+
     c = folder_name;
     while (*c != '\0') {
         if ((sep && *c == sep) || strchr ("%*", *c))
             return FALSE;
         c++;
     }
-    
+
     return TRUE;
 }
 
 
 static CamelFolderInfo *
-scalix_create_special_folder (CamelStore *store, 
-			      const char *parent_name, 
+scalix_create_special_folder (CamelStore *store,
+			      const char *parent_name,
 			      const char *folder_name,
 			      guint32 flags,
 			      CamelException *ex)
@@ -1062,65 +1046,68 @@ scalix_create_special_folder (CamelStore *store,
 	char *name;
 	char *type_str;
 	char sep;
-    const char *c;
+	const char *c;
 	int id;
 	guint32 type;
-	
+
 	sep = camel_scalix_get_path_delim (((CamelSCALIXStore *) store)->summary, parent_name);
-	
+
 	if (!scalix_check_foldername (folder_name, sep)) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_PATH,
 				      _("\"%s\" is not a valid foldername"), folder_name);
 		return NULL;
 	}
-	
+
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot create folders in offline mode."));
 		return NULL;
 	}
-	
+
 	if (parent_name != NULL && *parent_name)
 		name = g_strdup_printf ("%s/%s", parent_name, folder_name);
 	else
 		name = g_strdup (folder_name);
-	
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
 	utf7_name = scalix_folder_utf7_name (store, name, '\0');
-	
+
 	type = (flags & CAMEL_SCALIX_SFOLDER_MASK);
-	
+
 	if (type == CAMEL_SCALIX_FOLDER_CALENDAR) {
 		type_str = "IPF.Appointment";
 	} else if (type == CAMEL_SCALIX_FOLDER_CONTACT) {
-		type_str = "IPF.Contact";	
+		type_str = "IPF.Contact";
 	} else {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Unkown folder type"));
-		return NULL;	
+		return NULL;
 	}
-	
-	ic = camel_scalix_engine_queue (engine, NULL, 
-					"X-CREATE-SPECIAL %S %S\r\n", 
+
+	ic = camel_scalix_engine_queue (engine, NULL,
+					"X-CREATE-SPECIAL %S %S\r\n",
 					type_str, utf7_name);
 	g_free (utf7_name);
-	
+
 	while ((id = camel_scalix_engine_iterate (engine)) < ic->id && id != -1)
 		;
-	
+
 	if (id == -1 || ic->status != CAMEL_SCALIX_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
 		g_free (name);
 		goto done;
 	}
-	
+
 	switch (ic->result) {
 	case CAMEL_SCALIX_RESULT_OK:
 		url = camel_url_copy (engine->url);
 		url->path = g_strdup_printf ("/%s", name);
-		
+
 		c = strrchr (name, '/');
-		
+
 		fi = g_malloc0 (sizeof (CamelFolderInfo));
 		fi->full_name = name;
 		fi->name = g_strdup (c ? c + 1: name);
@@ -1129,9 +1116,9 @@ scalix_create_special_folder (CamelStore *store,
 		fi->flags = 0;
 		fi->unread = -1;
 		fi->total = -1;
-		
+
 		//camel_scalix_store_summary_note_info (((CamelSCALIXStore *) store)->summary, fi);
-		
+
 		//camel_object_trigger_event (store, "folder_created", fi);
 		break;
 	case CAMEL_SCALIX_RESULT_NO:
@@ -1150,17 +1137,20 @@ scalix_create_special_folder (CamelStore *store,
 	default:
 		g_assert_not_reached ();
 	}
-	
+
 	camel_scalix_command_unref (ic);
-	
+
  done:
-	
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
-	
+#endif
 	return fi;
 
 
-	return NULL;	
+	return NULL;
 }
 
 
@@ -1198,8 +1188,12 @@ scalix_create_folder (CamelStore *store, const char *parent_name, const char *fo
 	else
 		name = g_strdup (folder_name);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
+
 	utf7_name = scalix_folder_utf7_name (store, name, '\0');
 	ic = camel_scalix_engine_queue (engine, NULL, "CREATE %S\r\n", utf7_name);
 	g_free (utf7_name);
@@ -1255,8 +1249,11 @@ scalix_create_folder (CamelStore *store, const char *parent_name, const char *fo
 	
  done:
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
-	
+#endif	
 	return fi;
 }
 
@@ -1285,8 +1282,12 @@ scalix_delete_folder (CamelStore *store, const char *folder_name, CamelException
 		return;
 	}
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
+
 	if (selected && !strcmp (folder_name, selected->full_name))
 		ic0 = camel_scalix_engine_queue (engine, NULL, "CLOSE\r\n");
 	
@@ -1307,7 +1308,12 @@ scalix_delete_folder (CamelStore *store, const char *folder_name, CamelException
 			camel_scalix_command_unref (ic0);
 		
 		camel_scalix_command_unref (ic);
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 		return;
 	}
 	
@@ -1352,7 +1358,11 @@ scalix_delete_folder (CamelStore *store, const char *folder_name, CamelException
 	
 	camel_scalix_command_unref (ic);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 }
 
 
@@ -1440,8 +1450,12 @@ scalix_rename_folder (CamelStore *store, const char *old_name, const char *new_n
         return;
     }
 
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
+
 	old_uname = scalix_folder_utf7_name (store, old_name, '\0');
 	new_uname = scalix_folder_utf7_name (store, new_name, '\0');
 	
@@ -1459,7 +1473,11 @@ scalix_rename_folder (CamelStore *store, const char *old_name, const char *new_n
 	if (id == -1 || ic->status != CAMEL_SCALIX_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
+#if EAPI_CHECK_VERSION (2, 10)
+		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 		return;
 	}
 	
@@ -1499,7 +1517,11 @@ scalix_rename_folder (CamelStore *store, const char *old_name, const char *new_n
 	
 	camel_scalix_command_unref (ic);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 }
 
 static int
@@ -1811,8 +1833,12 @@ scalix_get_folder_info (CamelStore *store, const char *top, guint32 flags, Camel
 	else
 		base = top;
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
+
 #ifdef TRUE//USE_FOLDER_INFO_CACHE_LOGIC_FOR_SPEED
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL
 	    || engine->state == CAMEL_SCALIX_ENGINE_DISCONNECTED) {
@@ -1832,7 +1858,13 @@ scalix_get_folder_info (CamelStore *store, const char *top, guint32 flags, Camel
 			 * and query the server. */
 			goto check_online;
 		}
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
+
 		return fi;
 	}
 #else
@@ -1849,7 +1881,13 @@ scalix_get_folder_info (CamelStore *store, const char *top, guint32 flags, Camel
 				fi = inbox;
 			}
 		}
+
+#if EAPI_CHECK_VERSION (2, 10)
+		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
+
 		return fi;
 	}
 #endif
@@ -1951,8 +1989,11 @@ scalix_get_folder_info (CamelStore *store, const char *top, guint32 flags, Camel
 	
  done:
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
-	
+#endif	
 	return fi;
 }
 
@@ -1972,8 +2013,12 @@ scalix_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcept
 		return;
 	}
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif	
+
 	utf7_name = scalix_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_scalix_engine_queue (engine, NULL, "SUBSCRIBE %S\r\n", utf7_name);
 	g_free (utf7_name);
@@ -1984,7 +2029,13 @@ scalix_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcept
 	if (id == -1 || ic->status != CAMEL_SCALIX_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
+
 		return;
 	}
 	
@@ -2025,7 +2076,11 @@ scalix_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcept
 	
 	camel_scalix_command_unref (ic);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 }
 
 static void
@@ -2044,8 +2099,12 @@ scalix_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExce
 		return;
 	}
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif	
+
 	utf7_name = scalix_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_scalix_engine_queue (engine, NULL, "UNSUBSCRIBE %S\r\n", utf7_name);
 	g_free (utf7_name);
@@ -2056,7 +2115,11 @@ scalix_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExce
 	if (id == -1 || ic->status != CAMEL_SCALIX_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_scalix_command_unref (ic);
-		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 		return;
 	}
 	
@@ -2097,7 +2160,11 @@ scalix_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExce
 	
 	camel_scalix_command_unref (ic);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 }
 
 static void
@@ -2111,12 +2178,19 @@ scalix_noop (CamelStore *store, CamelException *ex)
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
 		return;
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif	
 	if (folder) {
 		camel_folder_sync (folder, FALSE, ex);
 		if (camel_exception_is_set (ex)) {
+#if EAPI_CHECK_VERSION (2, 10)
+			CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 			CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 			return;
 		}
 	}
@@ -2132,8 +2206,13 @@ scalix_noop (CamelStore *store, CamelException *ex)
 	
 	if (folder && !camel_exception_is_set (ex))
 		camel_scalix_summary_flush_updates (folder->summary, ex);
-	
+
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
+	
 }
 
 
@@ -2161,8 +2240,12 @@ change_password (CamelSCALIXStore *store, const char *str, CamelException *ex)
 	camel_url_decode (oldkey);
 	camel_url_decode (newkey);
 	
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	
+#endif
+
 	ic = camel_scalix_engine_queue (engine, 
 				       NULL, 
 				       "X-SCALIX-PASSWORD %s %s\r\n",
@@ -2210,7 +2293,11 @@ change_password (CamelSCALIXStore *store, const char *str, CamelException *ex)
 	}
 	
 	camel_scalix_command_unref (ic);
+#if EAPI_CHECK_VERSION (2, 10)
+	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+#else
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
+#endif
 }
 
 
