@@ -69,6 +69,8 @@ static int scalix_content_info_save (CamelFolderSummary *summary, FILE *out, Cam
 
 static CamelFolderSummaryClass *parent_class = NULL;
 
+static const char *scalix_message_info_get_user_tag (const CamelMessageInfo *mi, const char *tag);
+static gboolean    scalix_message_info_set_user_tag (CamelMessageInfo *info, const char *name, const char  *value);
 
 CamelType
 camel_scalix_summary_get_type (void)
@@ -105,6 +107,10 @@ camel_scalix_summary_class_init (CamelSCALIXSummaryClass *klass)
 	summary_class->message_info_clone = scalix_message_info_clone;
 	summary_class->content_info_load = scalix_content_info_load;
 	summary_class->content_info_save = scalix_content_info_save;
+
+	summary_class->info_user_tag = scalix_message_info_get_user_tag;
+	summary_class->info_set_user_tag = scalix_message_info_set_user_tag;
+
 }
 
 static void
@@ -429,32 +435,45 @@ envelope_decode_nstring (CamelSCALIXEngine *engine, char **nstring, gboolean rfc
 }
 
 static CamelSummaryReferences *
-decode_references (const char *string, int inreplyto)
+decode_references (const char *refstr, const char *irtstr)
 {
-	struct _camel_header_references *refs, *r;
+	struct _camel_header_references *refs, *irt, *r;
 	CamelSummaryReferences *references;
 	unsigned char md5sum[16];
 	guint32 i, n;
-	
-	if (inreplyto) {
-		if (!(refs = camel_header_references_inreplyto_decode (string)))
-			return NULL;
-	} else {
-		if (!(refs = camel_header_references_decode (string)))
-			return NULL;
+
+	refs = camel_header_references_decode (refstr);
+	irt = camel_header_references_inreplyto_decode (irtstr);
+
+	if (!refs && !irt)
+		return NULL;
+
+	if (irt) {
+		/* The References field is populated from the `References' and/or `In-Reply-To'
+		   headers. If both headers exist, take the first thing in the In-Reply-To header
+		   that looks like a Message-ID, and append it to the References header. */
+
+		if (refs) {
+			r = irt;
+			while (r->next != NULL)
+				r = r->next;
+			r->next = refs;
+		}
+
+		refs = irt;
 	}
-	
+
 	n = camel_header_references_list_size (&refs);
 	references = g_malloc (sizeof (CamelSummaryReferences) + (sizeof (CamelSummaryMessageID) * (n - 1)));
 	references->size = n;
-	
+
 	for (i = 0, r = refs; r != NULL; i++, r = r->next) {
 		md5_get_digest (r->id, strlen (r->id), md5sum);
-		memcpy (references->references[i].id.hash, md5sum, sizeof (references->references[i].id.hash));
+		memcpy (references->references[i].id.hash, md5sum, sizeof (CamelSummaryMessageID));
 	}
-	
+
 	camel_header_references_list_clear (&refs);
-	
+
 	return references;
 }
 
@@ -462,97 +481,97 @@ static int
 decode_envelope (CamelSCALIXEngine *engine, CamelMessageInfo *info, camel_scalix_token_t *token, CamelException *ex)
 {
 	CamelSCALIXMessageInfo *iinfo = (CamelSCALIXMessageInfo *) info;
-	CamelSummaryReferences *refs;
 	unsigned char md5sum[16];
-	char *nstring;
-	
+	char *nstring, *msgid;
+
 	if (camel_scalix_engine_next_token (engine, token, ex) == -1)
 		return -1;
-	
+
 	if (token->token != '(') {
 		camel_scalix_utils_set_unexpected_token_error (ex, engine, token);
 		return -1;
 	}
-	
+
 	if (envelope_decode_date (engine, &iinfo->info.date_sent, ex) == -1)
 		goto exception;
-	
+
 	/* subject */
 	if (envelope_decode_nstring (engine, &nstring, TRUE, ex) == -1)
 		goto exception;
 	iinfo->info.subject = camel_pstring_strdup (nstring);
 	g_free(nstring);
-	
+
 	/* from */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	iinfo->info.from = camel_pstring_strdup (nstring);
 	g_free(nstring);
-	
+
 	/* sender */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	g_free (nstring);
-	
+
 	/* reply-to */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	g_free (nstring);
-	
+
 	/* to */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	iinfo->info.to = camel_pstring_strdup (nstring);
 	g_free(nstring);
-	
+
 	/* cc */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	iinfo->info.cc = camel_pstring_strdup (nstring);
 	g_free(nstring);
-	
+
 	/* bcc */
 	if (envelope_decode_addresses (engine, &nstring, ex) == -1)
 		goto exception;
 	g_free (nstring);
-	
+
 	/* in-reply-to */
 	if (envelope_decode_nstring (engine, &nstring, FALSE, ex) == -1)
 		goto exception;
-	
+
 	if (nstring != NULL) {
-		refs = decode_references (nstring, TRUE);
-		
 		if (!iinfo->info.references)
-			iinfo->info.references = refs;
-		else
-			g_free (refs);
-		
+			iinfo->info.references = decode_references (NULL, nstring);
+
 		g_free (nstring);
 	}
-	
+
 	/* message-id */
 	if (envelope_decode_nstring (engine, &nstring, FALSE, ex) == -1)
 		goto exception;
-	
+
 	if (nstring != NULL) {
-		md5_get_digest (nstring, strlen (nstring), md5sum);
-		memcpy (iinfo->info.message_id.id.hash, md5sum, sizeof (iinfo->info.message_id.id.hash));
+
+		if ((msgid = camel_header_msgid_decode (nstring))) {
+			md5_get_digest (msgid, strlen (msgid), md5sum);
+			memcpy (iinfo->info.message_id.id.hash, md5sum, sizeof (CamelSummaryMessageID));
+			g_free (msgid);
+		}
+
 		g_free (nstring);
 	}
-	
+
 	if (camel_scalix_engine_next_token (engine, token, ex) == -1)
 		return -1;
-	
+
 	if (token->token != ')') {
 		camel_scalix_utils_set_unexpected_token_error (ex, engine, token);
 		goto exception;
 	}
-	
+
 	return 0;
-	
+
  exception:
-	
+
 	return -1;
 }
 
@@ -989,25 +1008,25 @@ untagged_fetch_all (CamelSCALIXEngine *engine, CamelSCALIXCommand *ic, guint32 i
 			struct _camel_header_raw *h;
 			CamelMimeParser *parser;
 			unsigned char *literal;
-			const char *str;
+			const char *refs, *str;
 			char *mlist;
 			size_t n;
-			
+
 			/* '(' */
 			if (camel_scalix_engine_next_token (engine, token, ex) == -1)
 				goto exception;
-			
+
 			if (token->token != '(')
 				goto unexpected;
-			
+
 			/* header name list */
 			do {
 				if (camel_scalix_engine_next_token (engine, token, ex) == -1)
 					goto exception;
-				
+
 				if (token->token == ')')
 					break;
-				
+
 				switch (token->token) {
 				case CAMEL_SCALIX_TOKEN_ATOM:
 				case CAMEL_SCALIX_TOKEN_QSTRING:
@@ -1063,10 +1082,11 @@ untagged_fetch_all (CamelSCALIXEngine *engine, CamelSCALIXCommand *ic, guint32 i
 				}
 				
 				/* check for References: */
-				if ((str = camel_header_raw_find (&h, "References", NULL))) {
-					g_free (iinfo->info.references);
-					iinfo->info.references = decode_references (str, FALSE);
-				}
+				g_free (iinfo->info.references);
+				refs = camel_header_raw_find (&h, "References", NULL);
+				str = camel_header_raw_find (&h, "In-Reply-To", NULL);
+				iinfo->info.references = decode_references (refs, str);
+
 			default:
 				break;
 			}
@@ -1102,7 +1122,7 @@ untagged_fetch_all (CamelSCALIXEngine *engine, CamelSCALIXCommand *ic, guint32 i
 #define SCALIX_ALL "FLAGS INTERNALDATE RFC822.SIZE ENVELOPE"
 #define MAILING_LIST_HEADERS "List-Post List-Id Mailing-List Originator X-Mailing-List X-Loop X-List Sender Delivered-To Return-Path X-BeenThere List-Unsubscribe"
 
-#define BASE_HEADER_FIELDS "Content-Type References"
+#define BASE_HEADER_FIELDS "Content-Type References In-Reply-To"
 #define MORE_HEADER_FIELDS BASE_HEADER_FIELDS " " MAILING_LIST_HEADERS
 
 static CamelSCALIXCommand *
@@ -1196,6 +1216,7 @@ scalix_message_info_new_from_header (CamelFolderSummary *summary, struct _camel_
 	return info;
 }
 
+
 static CamelMessageInfo *
 scalix_message_info_load (CamelFolderSummary *summary, FILE *fin)
 {
@@ -1209,7 +1230,7 @@ scalix_message_info_load (CamelFolderSummary *summary, FILE *fin)
 	
 	if (camel_file_util_decode_uint32 (fin, &minfo->server_flags) == -1)
 		goto exception;
-	
+		
 	return info;
 	
  exception:
@@ -1483,3 +1504,48 @@ camel_scalix_summary_flush_updates (CamelFolderSummary *summary, CamelException
 	
 	return 0;
 }
+
+static const char *
+scalix_message_info_get_user_tag (const CamelMessageInfo *mi, const char *id)
+{
+	const char *res = NULL;
+	
+	if (g_str_equal (id, "label")) {
+		const CamelSCALIXMessageInfo *iinfo = (const CamelSCALIXMessageInfo *) mi;
+
+		if ((iinfo->info.flags & CAMEL_SCALIX_MESSAGE_LABEL_MASK) == 0) {
+			return NULL;
+		}
+
+		res = scalix_flag_to_user_tag (iinfo->info.flags);
+				
+	} else {
+	
+		res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_user_tag (mi, id);
+	}
+
+	return res;
+}
+
+
+static gboolean
+scalix_message_info_set_user_tag (CamelMessageInfo *info, const char *name, const char *value)
+{
+	int res;
+
+	/* Let's only save flags not tags for the label case */
+	if (g_str_equal (name, "label")) {
+		guint32 flag;
+
+		flag = scalix_user_tag_to_flag (value);
+		
+		res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags (info,
+										 CAMEL_SCALIX_MESSAGE_LABEL_MASK,
+										 flag);
+	} else {
+	
+		res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_user_tag (info, name, value);
+	}
+
+	return res;
+}
