@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * Authors: Christian.Kellner@scalix.com    
+ * Authors: Christian.Kellner@scalix.com
  */
 
 #ifdef HAVE_CONFIG_H
@@ -70,7 +70,7 @@ static void set_property (GObject * object,
                           const GValue * value, GParamSpec * pspec);
 static void get_property (GObject * object,
                           guint prop_id, GValue * value, GParamSpec * pspec);
-
+static char * url_set_path_and_get_string (CamelURL *url, const char *path);
 /* ************************************************************************** */
 
 /* GObject Stuff */
@@ -338,11 +338,22 @@ create_base_uri (EUri * uri)
     user = camel_url_encode (uri->user, "@");
 
     if (uri->port != 0) {
-        uri_string = g_strdup_printf ("%s://%s@%s:%d/", uri->protocol,
-                                      user, uri->host, uri->port);
+        uri_string = g_strdup_printf ("%s://%s%s%s%s%s:%d/",
+				      uri->protocol,
+                                      user,
+				      uri->authmech ? ";auth=" : "",
+				      uri->authmech ? uri->authmech : "",
+				      uri->user ? "@" : "",
+				      uri->host,
+				      uri->port);
     } else {
-        uri_string = g_strdup_printf ("%s://%s@%s/", uri->protocol,
-                                      user, uri->host);
+        uri_string = g_strdup_printf ("%s://%s%s%s%s%s/",
+				      uri->protocol,
+                                      user,
+				      uri->authmech ? ";auth=" : "",
+				      uri->authmech ? uri->authmech : "",
+				      uri->user ? "@" : "",
+				      uri->host);
     }
 
     g_free (user);
@@ -400,13 +411,14 @@ sync_source_group (EAccount * account,
         CamelURL *url;
         CamelFolderInfo *info;
         const char *fuid;
+	const char *pssl;
         char *uid;
         xmlNodePtr node;
 
         info = (CamelFolderInfo *) iter->data;
 
         url = camel_url_new (info->uri, NULL);
-
+	g_print ("FOLDER_URI: %s\n", info->uri);
         if (url == NULL) {
             camel_url_free (url);
             continue;
@@ -422,7 +434,7 @@ sync_source_group (EAccount * account,
         }
 
         uid = g_strdup_printf ("%s@%s", fuid, account->uid);
-        camel_url_free (url);
+
 
         /* there is no such thing as e_source_set_uid 
          * so we have to cheat here a bit */
@@ -439,10 +451,16 @@ sync_source_group (EAccount * account,
 
         if (source == NULL) {
             g_warning ("Invalid source, skipping");
+	    camel_url_free (url);
             /* FIXME free resources */
             continue;
         }
 
+	pssl = camel_url_get_param (url, "use_ssl");
+	if (pssl != NULL) {
+		e_source_set_property (source, "use_ssl", pssl);
+	}
+
         if (info->flags & CAMEL_FOLDER_IS_DEFAULT) {
             e_source_set_property (source, "default", "TRUE");
 
@@ -451,6 +469,7 @@ sync_source_group (EAccount * account,
             }
         }
 
+	camel_url_free (url);
         e_source_group_add_source (group, source, -1);
     }
 }
@@ -483,6 +502,7 @@ create_ldap_source (EAccount * account, ESourceGroup * group)
     }
 
     g_free (uri);
+    g_free (uid);
     e_uri_free (euri);
 }
 
@@ -634,13 +654,12 @@ synch_data (ScalixAccountSynch * sxas,
             CamelFolderInfo * ftree, int type, guint prg_offset)
 {
     ScalixAccountSynchPrivate *priv;
-    EUri *euri;
     GSList *def;
     GSList *list;
     GSList *iter;
     guint n, i, prg;
+    CamelURL *url;
     const char *uri;
-    char *user;
 
     priv = SCALIX_ACCOUNT_SYNCH_GET_PRIVATE (sxas);
 
@@ -649,16 +668,12 @@ synch_data (ScalixAccountSynch * sxas,
     }
 
     uri = e_account_get_string (priv->account, E_ACCOUNT_SOURCE_URL);
-    euri = e_uri_new (uri);
+	url = camel_url_new (uri, NULL);
 
-    if (euri == NULL) {
+    if (url == NULL) {
         return FALSE;
     }
 
-    user = camel_url_encode (euri->user, "@");
-    g_free (euri->user);
-    euri->user = user;
-
     list = folder_tree_flatten (ftree, type, &def);
     n = g_slist_length (list);
 
@@ -670,8 +685,7 @@ synch_data (ScalixAccountSynch * sxas,
 
         fi = (CamelFolderInfo *) iter->data;
 
-        e_uri_set_path (euri, fi->full_name);
-        curi = e_uri_to_string (euri, FALSE);
+		curi = url_set_path_and_get_string (url, fi->full_name);
 
         prg = prg_offset + ((guint) ((((gdouble) i) / ((gdouble) n)) * 30.0));
 
@@ -680,18 +694,17 @@ synch_data (ScalixAccountSynch * sxas,
         g_free (msg);
 
         container = scalix_container_open (curi);
-
         g_free (curi);
 
         if (container == NULL) {
-            g_print ("SxAS: Could not open container\n");
+            g_debug ("SxAS: Could not open container\n");
             continue;
         }
 
         g_object_set (container, "online", TRUE, NULL);
         g_object_get (container, "items", &(priv->items), NULL);
+
         priv->items_done = 0;
-        g_print ("Items: %d\n", priv->items);
 
         g_signal_connect (container,
                           "object_added",
@@ -712,12 +725,35 @@ synch_data (ScalixAccountSynch * sxas,
 
     }
 
-    e_uri_free (euri);
-    return TRUE;
+	camel_url_free (url);
+	return TRUE;
+}
+
+static char *
+url_set_path_and_get_string (CamelURL *url, const char *path)
+{
+	char *url_str;
+	char *tmp = NULL;
+
+	if (!path || !path[0]) {
+		return NULL;
+	}
+
+	if (path[0] != '/') {
+		tmp = g_strconcat ("/", path, NULL);
+		path = tmp;
+	}
+
+	camel_url_set_path (url, path);
+	url_str = camel_url_to_string (url, 0);
+
+	g_free (tmp);
+
+	return url_str;
 }
 
 /* The worker thread function with all the complex
- * logic in it! Big fat beast! Please watch your 
+ * logic in it! Big fat beast! Please watch your
  * children ... */
 static void
 synch_worker (struct _mail_msg *mm)
@@ -823,17 +859,17 @@ synch_worker (struct _mail_msg *mm)
 
     /* Sent Items and Drafts Folder */
     if (priv->synch_dfolder) {
-        EUri *euri;
         char *url_str;
+	CamelURL *url;
+
+	url = camel_url_new (uri, NULL);
 
         signal_progress (sxas, 95, _("Synchronizing additional data"));
 
-        euri = e_uri_new (uri);
         sf = folder_tree_find_special (ftree, CAMEL_SCALIX_FOLDER_SENT);
 
-        if (euri && sf) {
-            e_uri_set_path (euri, sf->full_name);
-            url_str = e_uri_to_string (euri, FALSE);
+        if (url && sf) {
+	    url_str = url_set_path_and_get_string (url, sf->full_name);
 
             e_account_set_string (priv->account,
                                   E_ACCOUNT_SENT_FOLDER_URI, url_str);
@@ -842,26 +878,24 @@ synch_worker (struct _mail_msg *mm)
 
         sf = folder_tree_find_special (ftree, CAMEL_SCALIX_FOLDER_DRAFTS);
 
-        if (euri && sf) {
-            e_uri_set_path (euri, sf->full_name);
-            url_str = e_uri_to_string (euri, FALSE);
-
+        if (url && sf) {
+	    url_str = url_set_path_and_get_string (url, sf->full_name);
             e_account_set_string (priv->account,
                                   E_ACCOUNT_DRAFTS_FOLDER_URI, url_str);
 
             g_free (url_str);
         }
 
-        if (euri) {
+        if (url) {
             gboolean save_pw;
 
-            e_uri_set_path (euri, NULL);
+            camel_url_set_path (url, NULL);
 
-            if (euri->authmech == NULL) {
-                euri->authmech = g_strdup ("PLAIN");
+            if (url->authmech == NULL) {
+		    camel_url_set_authmech (url, "PLAIN");
             }
 
-            url_str = e_uri_to_string (euri, FALSE);
+	    url_str = camel_url_to_string (url, 0);
             e_account_set_string (priv->account,
                                   E_ACCOUNT_TRANSPORT_URL, url_str);
 
@@ -872,8 +906,8 @@ synch_worker (struct _mail_msg *mm)
                                 E_ACCOUNT_TRANSPORT_SAVE_PASSWD, save_pw);
         }
 
-        if (euri) {
-            e_uri_free (euri);
+        if (url) {
+		camel_url_free (url);
         }
     }
 
