@@ -39,6 +39,10 @@
 #include "db-local.h"
 #endif
 
+#if DB_VERSION_MINOR == 4
+#define ERROR_REPORT_3ARGS 1
+#endif
+
 #include "scalix-object.h"
 #include "scalix-object-cache.h"
 #include "scalix-utils.h"
@@ -209,7 +213,7 @@ scalix_object_cache_init (ScalixObjectCache *cache)
 
 /* db helper functions */
 
-#if (DB_VERSION_MINOR == 3)
+#ifdef ERROR_REPORT_3ARGS
 static void
 db_error_report (const DB_ENV *db_env, const char *buf1, const char *buf2)
 {
@@ -782,6 +786,7 @@ scalix_object_cache_put (ScalixObjectCache *cache,
 	
 	g_free (ouid);
 	g_free (data);
+	g_free (payload);
 	
 	if (db_res != 0) {
 		GLOG_CAT_WARNING (&sloc, "Could not commit transaction");
